#!/bin/bash
#
##
# clpgcdns-watch.sh
##
# monitoring the clpgcdns
##


echo "clpgcdns watch script start."


##
# ret code
##
GCDNSW_SUCCESS_CODE=0					# succeed
GCDNSW_ERR_GET_VALUE_CODE=50			# failed to get the setting value
GCDNSW_ERR_GET_ZONENAME_CODE=51			# get zone name has failed
GCDNSW_ERR_GET_RECORDNAME_CODE=52		# get record name has failed
GCDNSW_ERR_GET_RECORDTYPE_CODE=53		# get record type has failed
GCDNSW_ERR_GET_RECORDTTL_CODE=54		# get ttl has failed
GCDNSW_ERR_GET_RECORDIP_CODE=55			# get ip address has failed
GCDNSW_ERR_GET_RECORD_CODE=56			# get dns record has failed
GCDNSW_ERR_NO_RECORD_CODE=57			# there is no designated dns record


##
# resultmsg
##
GCDNSW_ERR_GET_VALUE_MSG="Failed to obtain the setting value."
GCDNSW_ERR_GET_ZONENAME_MSG="Failed to obtain the zone name."
GCDNSW_ERR_GET_RECORDNAME_MSG="Failed to obtain the DNS name."
GCDNSW_ERR_GET_RECORDTYPE_MSG="Failed to obtain the record type."
GCDNSW_ERR_GET_RECORDTTL_MSG="Failed to obtain the TTL."
GCDNSW_ERR_GET_RECORDIP_MSG="Failed to obtain the IP address."
GCDNSW_ERR_GET_RECORD_MSG="Failed to obtain the record set."
GCDNSW_ERR_NO_RECORD_MSG="No record set to be monitored."


##
# set the value to use for the shmrmset command
##
RESOURCE_NAME="${CLP_RESOURCENAME}"		# resource name
RESOURCE_TYPE="gcdnsw"						# resource type

if [ -z "${RESOURCE_NAME}" ]
then
	echo "failed to obtain the value required for the shmrmset command." 1>&2
fi


##
# set value
##
TARGET_RSC="${CLP_OCF_PARAM1}"			# resource name
SRV_FULLNAME="`uname -n`"				# get hostname (use output file path)
SRV_NAME="`echo ${SRV_FULLNAME%%.*}`"	# delete domain name

if [ -z "$TARGET_RSC" ]
then
	clpshmrmset --descript --mon -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${GCDNSW_ERR_GET_VALUE_MSG}" > /dev/null 2>&1
	echo "failed to get the setting value." 1>&2
	echo "clpgcdns watch script has failed." 1>&2
	exit ${GCDNSW_ERR_GET_VALUE_CODE}
fi

##
# get target resource config
##
ZONE_NAME="`clpcfget -g /root/resource/gcdns@\"${TARGET_RSC}\"/parameters/zone_name -p gcdns`"
if [ $? -ne 0 ]
then
	clpshmrmset --descript --mon -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${GCDNSW_ERR_GET_ZONENAME_MSG}" > /dev/null 2>&1
	echo "$ZONE_NAME" | head -n 1 1>&2
	echo "get zone name has failed." 1>&2
	echo "clpgcdns watch script has failed." 1>&2
	exit ${GCDNSW_ERR_GET_ZONENAME_CODE}
fi

RECORD_NAME="`clpcfget -g /root/resource/gcdns@\"${TARGET_RSC}\"/parameters/dns_name -p gcdns`"
if [ $? -ne 0 ]
then
	clpshmrmset --descript --mon -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${GCDNSW_ERR_GET_RECORDNAME_MSG}" > /dev/null 2>&1
	echo "$RECORD_NAME" | head -n 1 1>&2
	echo "get record name has failed." 1>&2
	echo "clpgcdns watch script has failed." 1>&2
	exit ${GCDNSW_ERR_GET_RECORDNAME_CODE}
fi

RECORD_TYPE="`clpcfget -g /root/resource/gcdns@\"${TARGET_RSC}\"/parameters/record_type -p gcdns`"
if [ $? -ne 0 ]
then
	clpshmrmset --descript --mon -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${GCDNSW_ERR_GET_RECORDTYPE_MSG}" > /dev/null 2>&1
	echo "$RECORD_TYPE" | head -n 1 1>&2
	echo "get record type has failed." 1>&2
	echo "clpgcdns watch script has failed." 1>&2
	exit ${GCDNSW_ERR_GET_RECORDTYPE_CODE}
fi

RECORD_TTL="`clpcfget -g /root/resource/gcdns@\"${TARGET_RSC}\"/parameters/record_ttl -p gcdns`"
if [ $? -ne 0 ]
then
	clpshmrmset --descript --mon -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${GCDNSW_ERR_GET_RECORDTTL_MSG}" > /dev/null 2>&1
	echo "$RECORD_TTL" | head -n 1 1>&2
	echo "get ttl has failed." 1>&2
	echo "clpgcdns watch script has failed." 1>&2
	exit ${GCDNSW_ERR_GET_RECORDTTL_CODE}
fi

RECORD_IP="`clpcfget -g /root/resource/gcdns@\"${TARGET_RSC}\"/parameters/record_ip -p gcdns -s \"${SRV_NAME}\"`"
if [ $? -ne 0 ]
then
	clpshmrmset --descript --mon -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${GCDNSW_ERR_GET_RECORDIP_MSG}" > /dev/null 2>&1
	echo "$RECORD_IP" | head -n 1 1>&2
	echo "get ip address has failed." 1>&2
	echo "clpgcdns watch script has failed." 1>&2
	exit ${GCDNSW_ERR_GET_RECORDIP_CODE}
fi

echo "get config has succeeded (zone=${ZONE_NAME} name=${RECORD_NAME} type=${RECORD_TYPE} ttl=${RECORD_TTL} ip=${RECORD_IP})."


##
# get dns record
##
CURRENT="`gcloud dns record-sets list -z=\"${ZONE_NAME}\" --name=\"${RECORD_NAME}\" --type=\"${RECORD_TYPE}\" \
	--format=\"table[no-heading](name,type,ttl, rrdatas.list():label=DATA)\"`"
if [ $? -ne 0 ]
then
	clpshmrmset --descript --mon -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "$GCDNSW_ERR_GET_RECORD_MSG" > /dev/null 2>&1
	echo "get dns record has failed." 1>&2
	echo "clpgcdns watch script has failed." 1>&2
	exit ${GCDNSW_ERR_GET_RECORD_CODE}
elif [ -z "$CURRENT" ]
then
	clpshmrmset --descript --mon -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${GCDNSW_ERR_NO_RECORD_MSG} (get dns record)" > /dev/null 2>&1
	echo "there is no designated dns record." 1>&2
	echo "clpgcdns watch script has failed." 1>&2
	exit ${GCDNSW_ERR_NO_RECORD_CODE}
fi


##
# ttl ip check
##
CURRENT_TTL="`echo ${CURRENT} | awk '{print $3}'`"
CURRENT_IP="`echo ${CURRENT} | awk '{print $4}'`"

if [ "$CURRENT_TTL" -ne "$RECORD_TTL" ]
then
	clpshmrmset --descript --mon -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${GCDNSW_ERR_NO_RECORD_MSG} (ttl check)" > /dev/null 2>&1
	echo "CURRENT_TTL = ${CURRENT_TTL} / RECORD_TTL = ${RECORD_TTL}" 1>&2
	echo "ttl is not suitable." 1>&2
	echo "clpgcdns watch script has failed." 1>&2
	exit ${GCDNSW_ERR_NO_RECORD_CODE}
fi

if [ "$CURRENT_IP" != "$RECORD_IP" ]
then
	clpshmrmset --descript --mon -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${GCDNSW_ERR_NO_RECORD_MSG} (ip check)" > /dev/null 2>&1
	echo "CURRENT_IP = ${CURRENT_IP} / RECORD_IP = ${RECORD_IP}" 1>&2
	echo "ip address is not suitable." 1>&2
	echo "clpgcdns watch script has failed." 1>&2
	exit ${GCDNSW_ERR_NO_RECORD_CODE}
fi

##
# end
##
echo "clpgcdns watch script has succeeded."

exit ${GCDNSW_SUCCESS_CODE}

